﻿/**
 * AGreement Analysis Toolkit, AGATe v2.0 (.NET Framework 4.5 C# version)
 * 
 * 	    Radu-Daniel Vatavu, Ph.D.
 * 	    MintViz Lab | MANSiD Research Center
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 * 	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publications for the Agreement Analysis toolkit, and what should be 
 * used to cite it, are:
 *
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2016). 
 *  Between-subjects elicitation studies: Formalization and tool support.
 *  In Proceedings of CHI '16, the 34th ACM Conference on Human Factors in Computing Systems
 *  (San Jose, California, USA, May 2016). New York: ACM Press.
 *	
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2015). 
 *  Formalizing Agreement Analysis for Elicitation Studies: New Measures, Significance Test, and Toolkit.
 *  In Proceedings of CHI'15, the 33rd ACM SIGCHI Conference on Human Factors in Computing Systems 
 *  (Seoul, Republic of Korea, April 2015). New York: ACM Press.
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2015-2016, Radu-Daniel Vatavu and Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, nor 
 *	    University of Washington, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Jacob O. Wobbrock 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using AgreementRates;

namespace AGATe
{
    public partial class _2RelatedSamplesForm : Form
    {
        struct Pair
        {
            public int Index1, Index2;
        };
        private List<Pair> selectedPairs = null;
        private TextBox tbLog = null;
        private List<string[]> proposalsPerReferent = null;

        #region form constructor

        public _2RelatedSamplesForm(List<string> referentNames, List<string[]> proposalsPerReferent, TextBox tbLog)
        {
            InitializeComponent();
            
            this.tbLog = tbLog;
            this.proposalsPerReferent = proposalsPerReferent;
            for (int i = 0; i < referentNames.Count; i++)
                lbReferents.Items.Add(referentNames[i]);
            selectedPairs = new List<Pair>();
        }

        #endregion

        #region form actions: add pair of referents, remove pair, cancel

        private void btnAddPair_Click(object sender, EventArgs e)
        {
            if (lbReferents.CheckedIndices.Count != 2)
            {
                MessageBox.Show("Please select a pair of two referents.", "Error");
                return;
            }

            string pair = String.Format("{0} vs. {1}", lbReferents.CheckedItems[0], lbReferents.CheckedItems[1]);
            lbPairs.Items.Add(pair);
            selectedPairs.Add(new Pair() 
                { 
                    Index1 = lbReferents.CheckedIndices[0],
                    Index2 = lbReferents.CheckedIndices[1] 
                });
            lbReferents.SetItemCheckState(lbReferents.CheckedIndices[0], CheckState.Unchecked);
            lbReferents.SetItemCheckState(lbReferents.CheckedIndices[0], CheckState.Unchecked);
        }

        private void btnRemovePair_Click(object sender, EventArgs e)
        {
            if (lbPairs.SelectedIndex >= 0)
            {
                selectedPairs.RemoveAt(lbPairs.SelectedIndex);
                lbPairs.Items.RemoveAt(lbPairs.SelectedIndex);
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion

        #region run test statistic

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (selectedPairs.Count == 0)
                return;

            tbLog.Text += "-------------------------------------------------\r\n";
            tbLog.Text += ">> Computing Vrd statistic (2 related samples)...\r\n";
            tbLog.Text += "-------------------------------------------------\r\n";

            foreach (Pair pair in selectedPairs)
            {
                tbLog.Text += String.Format("AR({0}) = {1:.000}\r\n", lbReferents.Items[pair.Index1], AgreementMeasures.AR(proposalsPerReferent[pair.Index1]));
                tbLog.Text += String.Format("AR({0}) = {1:.000}\r\n", lbReferents.Items[pair.Index2], AgreementMeasures.AR(proposalsPerReferent[pair.Index2]));
                double p;
                double V = AgreementTestsOfSignificance.Vrd(proposalsPerReferent[pair.Index1], proposalsPerReferent[pair.Index2], out p);
                tbLog.Text += String.Format("Vrd(1,N={0}) = {1:.000}, p = {2:.000}\r\n\r\n", 
                    proposalsPerReferent[pair.Index1].Length + proposalsPerReferent[pair.Index2].Length,
                    V, p
                );
            }

            tbLog.Text += "Please apply Bonferroni corrections when interpreting these results.\r\n\r\n";
            tbLog.SelectionStart = tbLog.Text.Length;
            tbLog.ScrollToCaret();

            this.Close();
        }

        #endregion
    }
}
