﻿/**
 * AGreement Analysis Toolkit, AGATe v2.0 (.NET Framework 4.5 C# version)
 * 
 * 	    Radu-Daniel Vatavu, Ph.D.
 * 	    MintViz Lab | MANSiD Research Center
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 * 	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publications for the Agreement Analysis toolkit, and what should be 
 * used to cite it, are:
 *
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2016). 
 *  Between-subjects elicitation studies: Formalization and tool support.
 *  In Proceedings of CHI '16, the 34th ACM Conference on Human Factors in Computing Systems
 *  (San Jose, California, USA, May 2016). New York: ACM Press.
 *	
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2015). 
 *  Formalizing Agreement Analysis for Elicitation Studies: New Measures, Significance Test, and Toolkit.
 *  In Proceedings of CHI'15, the 33rd ACM SIGCHI Conference on Human Factors in Computing Systems 
 *  (Seoul, Republic of Korea, April 2015). New York: ACM Press.
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2015-2016, Radu-Daniel Vatavu and Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, nor 
 *	    University of Washington, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Jacob O. Wobbrock 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using AgreementRates;

namespace AGATe
{
    public partial class CoagreementForm : Form
    {
        private List<string[]> proposalsPerReferent = null;
        private TextBox tbLog = null;
     
        #region form constructor

        public CoagreementForm(List<string> referentNames, List<string[]> proposalsPerReferent, TextBox tbLog)
        {
            InitializeComponent();

            for (int i = 0; i < referentNames.Count; i++)
                lbReferents.Items.Add(referentNames[i]);
            this.proposalsPerReferent = proposalsPerReferent;
            this.tbLog = tbLog;
        }

        #endregion

        #region form controls and associated actions: select all referents, cancel

        private void cbSelectAll_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 0; i < lbReferents.Items.Count; i++)
                lbReferents.SetItemCheckState(i, cbSelectAll.CheckState);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion

        #region compute and display coagreement rates

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (lbReferents.CheckedIndices.Count < 2)
            {
                MessageBox.Show("Please select at least two referents.", "Error");
                return;
            }

            List<string[]> proposalsPerSelectedReferents = new List<string[]>();
            List<string> selectedReferentsNames = new List<string>();
            foreach (int index in lbReferents.CheckedIndices)
            {
                proposalsPerSelectedReferents.Add(proposalsPerReferent[index]);
                selectedReferentsNames.Add(lbReferents.Items[index].ToString());
            }

            tbLog.Text += "---------------------------------\r\n";
            tbLog.Text += ">> Computing coagreement rates...\r\n";
            tbLog.Text += "---------------------------------\r\n";
            if (cbOutputARValues.Checked)
            {
                for (int index = 0; index < selectedReferentsNames.Count; index++)
                    tbLog.Text += String.Format("AR({0}) = {1:.000}\r\n", selectedReferentsNames[index], AgreementMeasures.AR(proposalsPerSelectedReferents[index]));
            }

            // display overall coagreement
            tbLog.Text += "\r\n";
            tbLog.Text += String.Format("CR for all {0} referents = {1:.000}", proposalsPerSelectedReferents.Count, AgreementMeasures.CR(proposalsPerSelectedReferents));
            tbLog.Text += "\r\n\r\n";

            // display coagreement matrix
            tbLog.Text += new String(' ', 11);
            for (int j = 0; j < selectedReferentsNames.Count; j++)
                tbLog.Text += String.Format("{0,9} ", selectedReferentsNames[j].Substring(0, Math.Min(8, selectedReferentsNames[j].Length)));
            tbLog.Text += "\r\n";
            tbLog.Text += new String(' ', 11) + new String('-', 10 * selectedReferentsNames.Count) + "\r\n";

            for (int i = 0; i < selectedReferentsNames.Count; i++)
            {
                tbLog.Text += String.Format("{0,9} ", selectedReferentsNames[i].Substring(0, Math.Min(8, selectedReferentsNames[i].Length))) + "|";
                for (int j = 0; j < selectedReferentsNames.Count; j++)
                    tbLog.Text += String.Format("{0,9:.000} ", AgreementMeasures.CR(proposalsPerSelectedReferents[i], proposalsPerSelectedReferents[j]));
                tbLog.Text += "\r\n";
            }
            tbLog.Text += "\r\n";
            tbLog.SelectionStart = tbLog.Text.Length;
            tbLog.ScrollToCaret();

            this.Close();
        }

        #endregion
    }
}
