﻿/**
 * AGreement Analysis Toolkit, AGATe v2.0 (.NET Framework 4.5 C# version)
 * 
 * 	    Radu-Daniel Vatavu, Ph.D.
 * 	    MintViz Lab | MANSiD Research Center
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 * 	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publications for the Agreement Analysis toolkit, and what should be 
 * used to cite it, are:
 *
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2016). 
 *  Between-subjects elicitation studies: Formalization and tool support.
 *  In Proceedings of CHI '16, the 34th ACM Conference on Human Factors in Computing Systems
 *  (San Jose, California, USA, May 2016). New York: ACM Press.
 *	
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2015). 
 *  Formalizing Agreement Analysis for Elicitation Studies: New Measures, Significance Test, and Toolkit.
 *  In Proceedings of CHI'15, the 33rd ACM SIGCHI Conference on Human Factors in Computing Systems 
 *  (Seoul, Republic of Korea, April 2015). New York: ACM Press.
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2015-2016, Radu-Daniel Vatavu and Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, nor 
 *	    University of Washington, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Jacob O. Wobbrock 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using AgreementRates;

namespace AGATe
{
    public partial class GIndependentSamplesForm : Form
    {
        private List<string[]> proposalsPerReferent = null;
        private TextBox tbLog;

        #region form constructor

        public GIndependentSamplesForm(List<string> referentNames, List<string[]> proposalsPerReferent, TextBox tbLog)
        {
            InitializeComponent();

            for (int i = 0; i < referentNames.Count; i++)
                lbReferents.Items.Add(referentNames[i]);
            this.proposalsPerReferent = proposalsPerReferent;

            this.tbLog = tbLog;
            
            int numParticipants = proposalsPerReferent[0].Length;
            for (int i = 1; i <= numParticipants; i++)
                listParticipants.Items.Add("P" + i.ToString());
            btnRemoveGroup.Enabled = false;
        }

        #endregion

        #region form actions: manage groups, select all referents, cancel

        /// <summary>
        /// Selects all referents.
        /// </summary>
        private void cbSelectAll_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 0; i < lbReferents.Items.Count; i++)
                lbReferents.SetItemCheckState(i, cbSelectAll.CheckState);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Creates a group of participants.
        /// </summary>
        private void btnCreateGroup_Click(object sender, EventArgs e)
        {
            int numParticipantsInGroup = 0;
            for (int i = 0; i < listParticipants.CheckedIndices.Count; i++)
            {
                if (listParticipants.GetItemCheckState(listParticipants.CheckedIndices[i]) == CheckState.Indeterminate) continue;   // ignore participants that were selected previously (currently marked in the list as "indeterminate")
                numParticipantsInGroup++;
            }

            if (numParticipantsInGroup < 2)
                MessageBox.Show("Please select at least two participants to create a new group.", "Info");
            else
            {
                string group = "";
                for (int i = 0; i < listParticipants.CheckedIndices.Count; i++)
                {
                    if (listParticipants.GetItemCheckState(listParticipants.CheckedIndices[i]) == CheckState.Indeterminate) continue;   // ignore participants that were selected previously (currently marked in the list as "indeterminate")
                    group += String.Format("P{0} ", listParticipants.CheckedIndices[i] + 1);
                    listParticipants.SetItemCheckState(listParticipants.CheckedIndices[i], CheckState.Indeterminate);
                }
                if (group.Length > 0)
                {
                    listParticipants.SelectedItems.Clear();
                    listGroups.Items.Add(group);
                    btnRemoveGroup.Enabled = true;
                }
            }
        }

        /// <summary>
        /// Removes a group of participants.
        /// </summary>
        private void btnRemoveGroup_Click(object sender, EventArgs e)
        {
            if (listGroups.SelectedIndex == -1)
                MessageBox.Show("Please select a group of participants to remove.", "Info");
            else
            {
                string[] participantsIndexes = listGroups.SelectedItem.ToString().Trim().Split(new char[] { ' ' });
                listParticipants.ItemCheck -= listParticipants_ItemCheck;
                foreach (string participantIndex in participantsIndexes)
                {
                    int index = int.Parse(participantIndex.Substring(1)) - 1;
                    listParticipants.SetItemCheckState(index, CheckState.Unchecked);
                }
                listParticipants.ItemCheck += listParticipants_ItemCheck;
                listGroups.Items.RemoveAt(listGroups.SelectedIndex);
                listGroups.SelectedIndex = -1;

                if (listGroups.Items.Count == 0)
                    btnRemoveGroup.Enabled = false;
            }
        }

        /// <summary>
        /// Disables user selection of previously selected participants.
        /// </summary>
        private void listParticipants_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (e.CurrentValue == CheckState.Indeterminate)
            {
                e.NewValue = CheckState.Indeterminate;
                listParticipants.SetSelected(e.Index, false);
            }
        }

        #endregion

        #region compute and report the Vb test statistic

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (lbReferents.CheckedIndices.Count < 1)
            {
                MessageBox.Show("Please select at least one referent.", "Info");
                return;
            }

            // check if there are at least two groups
            if (listGroups.Items.Count < 2)
            {
                MessageBox.Show("Please create at least two groups of participants.", "Info");
                return;
            }

            // check if all participants have been assigned to groups
            for (int i = 0; i < listParticipants.Items.Count; i++)
                if (listParticipants.GetItemCheckState(i) != CheckState.Indeterminate)
                {
                    if (MessageBox.Show(String.Format("Not all participants have been assigned to groups\r\n(e.g., P{0} is not assigned to any group). Continue?", i + 1), "Info", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.No)
                        return;
                    else break;
                }

            tbLog.Text += "----------------------------------------------------\r\n";
            tbLog.Text += String.Format(">> Computing Vb statistic (k={0} independent groups)...\r\n", listGroups.Items.Count);
            tbLog.Text += "----------------------------------------------------\r\n";

            // compute the Vbg test statistic for each referent
            foreach (int indexReferent in lbReferents.CheckedIndices)
            {
                List<string> allProposals = new List<string>();
                List<string[]> groups = new List<string[]>();
                for (int indexGroup = 0; indexGroup < listGroups.Items.Count; indexGroup++)
                {
                    string[] participants = listGroups.Items[indexGroup].ToString().Trim().Split(new char[] { ' ' });
                    string[] proposalsPerParticipant = new string[participants.Length];
                    for (int i = 0; i < participants.Length; i++)
                    {
                        int indexParticipant = int.Parse(participants[i].Substring(1)) - 1;
                        proposalsPerParticipant[i] = proposalsPerReferent[indexReferent][indexParticipant];
                        allProposals.Add(proposalsPerParticipant[i]);
                    }
                    groups.Add(proposalsPerParticipant);
                }

                tbLog.Text += String.Format("Referent: {0} \r\n", lbReferents.Items[indexReferent].ToString());
                decimal p = 0;
                decimal Vb = AgreementTestsOfSignificance.Vb(groups, out p, false);
                tbLog.Text += String.Format("\r\nVbg({0},N={1}) = {2:.000}, p = {3:.000}\r\n", groups.Count, allProposals.Count, Vb, p);
                
                if (cbOutputPerGroupARValues.Checked)
                    for (int indexGroup = 0; indexGroup < groups.Count; indexGroup++)
                    {
                        double[] CI = null;
                        double AR = AgreementMeasures.AR(groups[indexGroup], out CI);
                        tbLog.Text += String.Format("\t Group #{0}: AR = {1:.000}, CI 95% = [{2:.000}, {3:.000}]\r\n", indexGroup + 1, AR, CI[0], CI[1]);
                    }

                if (cbOutputCoagreementBetweenGroups.Checked)
                {
                    double CRb = AgreementMeasures.CRb(groups);
                    string argumentsText = "";
                    for (int indexGroup = 0; indexGroup < groups.Count; indexGroup++)
                        argumentsText += String.Format("#{0}{1}", indexGroup + 1, indexGroup == groups.Count - 1 ? "" : ",");
                    tbLog.Text += String.Format("\t CRb({0}) = {1:.000}\r\n", argumentsText, CRb);
                }

                if (cbPosthocTests.Checked)
                {
                    tbLog.Text += "\r\nRunning Post-hoc tests...\r\n";
                    for (int indexGroup1 = 0; indexGroup1 < groups.Count; indexGroup1++)
                        for (int indexGroup2 = indexGroup1 + 1; indexGroup2 < groups.Count; indexGroup2++)
                        {
                            List<string[]> posthocGroups = new List<string[]>();
                            posthocGroups.Add(groups[indexGroup1]);
                            posthocGroups.Add(groups[indexGroup2]);

                            double CRb = cbOutputCoagreementBetweenGroups.Checked ? AgreementMeasures.CRb(posthocGroups) : 0.0;
                            
                            p = 0;
                            Vb = AgreementTestsOfSignificance.Vb(posthocGroups, out p, false);
                            tbLog.Text += String.Format("\t (#{0},#{1}): Vbg(2,N={2}) = {3:.000}, p = {4:.000} {5}\r\n",
                                indexGroup1 + 1, indexGroup2 + 1,
                                posthocGroups[0].Length + posthocGroups[1].Length,
                                Vb, p,
                                cbOutputCoagreementBetweenGroups.Checked ? String.Format("CRb(#{0},#{1}) = {2:.000}", indexGroup1 + 1, indexGroup2 + 1, CRb) : ""
                            );
                        }
                    tbLog.Text += "\r\nPlease apply Bonferroni corrections when interpreting results for post-hoc tests.\r\n\r\n";
                }
            }
            tbLog.SelectionStart = tbLog.Text.Length;
            tbLog.ScrollToCaret();
        }

        #endregion
    }
}
