﻿/**
 * AGreement Analysis Toolkit, AGATe v2.0 (.NET Framework 4.5 C# version)
 * 
 * 	    Radu-Daniel Vatavu, Ph.D.
 * 	    MintViz Lab | MANSiD Research Center
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 * 	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publications for the Agreement Analysis toolkit, and what should be 
 * used to cite it, are:
 *
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2016). 
 *  Between-subjects elicitation studies: Formalization and tool support.
 *  In Proceedings of CHI '16, the 34th ACM Conference on Human Factors in Computing Systems
 *  (San Jose, California, USA, May 2016). New York: ACM Press.
 *	
 *  Radu-Daniel Vatavu, Jacob O. Wobbrock. (2015). 
 *  Formalizing Agreement Analysis for Elicitation Studies: New Measures, Significance Test, and Toolkit.
 *  In Proceedings of CHI'15, the 33rd ACM SIGCHI Conference on Human Factors in Computing Systems 
 *  (Seoul, Republic of Korea, April 2015). New York: ACM Press.
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2015-2016, Radu-Daniel Vatavu and Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, nor 
 *	    University of Washington, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Jacob O. Wobbrock 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using AgreementRates;

namespace AGATe
{
    public partial class ZeroForm : Form
    {
        private List<string[]> proposalsPerReferent = null;
        private TextBox tbLog = null;

        #region form constructor

        public ZeroForm(List<string> referentNames, List<string[]> proposalsPerReferent, TextBox tbLog)
        {
            InitializeComponent();

            for (int i = 0; i < referentNames.Count; i++)
                lbReferents.Items.Add(referentNames[i]);
            this.proposalsPerReferent = proposalsPerReferent;
            this.tbLog = tbLog;
        }

        #endregion

        #region form actions: select all, cancel

        private void cbSelectAll_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 0; i < lbReferents.Items.Count; i++)
                lbReferents.SetItemCheckState(i, cbSelectAll.CheckState);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion

        #region run statistical test

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (lbReferents.CheckedIndices.Count == 0)
            {
                MessageBox.Show("Please select at least one referent.", "Error");
                return;
            }

            tbLog.Text += "---------------------------------------------------------\r\n";
            tbLog.Text += ">> Computing Vrd statistic (significance against zero)...\r\n";
            tbLog.Text += "---------------------------------------------------------\r\n";
            foreach (int index in lbReferents.CheckedIndices)
            {
                double p;
                double V = AgreementTestsOfSignificance.Vrd(proposalsPerReferent[index], out p);
                tbLog.Text += String.Format(
                    "AR({0}) = {1:.000}, Vrd(1,N={2}) = {3:.000}, p = {4:.000}\r\n", 
                    lbReferents.Items[index],
                    AgreementMeasures.AR(proposalsPerReferent[index]),
                    proposalsPerReferent[index].Length,
                    V, p);
            }
            tbLog.Text += "\r\n";
            tbLog.SelectionStart = tbLog.Text.Length;
            tbLog.ScrollToCaret();

            this.Close();
        }

        #endregion
    }
}
